// Generated by CoffeeScript 2.4.0
var DEFAULTS, SKIP, combineReducers, commonReducer, createActions, createReducer, identity, initialReducer, isFunction, reducerEnhancer, skipMiddleware, toSnakeCase, wrapCreator;

SKIP = {
  type: null
};

DEFAULTS = {
  JOIN: '/'
};

identity = function(payload) {
  return payload;
};

isFunction = function(v) {
  return typeof v === 'function';
};

toSnakeCase = function(s) {
  var c, i, len, upperCharts;
  s = s.toString();
  upperCharts = s.match(/[A-Z]/g);
  if (!upperCharts) {
    return s;
  }
  for (i = 0, len = upperCharts.length; i < len; i++) {
    c = upperCharts[i];
    s = s.replace(new RegExp(`${c}`), `_${c.toLowerCase()}`);
  }
  if (s[0] === '_') {
    return s.slice(1);
  } else {
    return s;
  }
};

// Wrap results into actions with types and payloads
wrapCreator = function(type, creator) {
  return function(...args) {
    var action, m;
    action = creator(...args);
    if (!action) {
      return {type};
    }
    if (action === SKIP) {
      return SKIP;
    }
    // Support redux-thunk middleware
    if (isFunction(action)) {
      m = wrapCreator(type, action);
      return function(dispatch, ...args) {
        return dispatch(m(dispatch, ...args));
      };
    }
    // Support promises (requires redux-thunk)
    if (isFunction(action.then)) {
      m = wrapCreator(type, identity);
      return function(dispatch) {
        return action.then(function(a) {
          dispatch(m(a));
          return a;
        });
      };
    }
    if (action.type) {
      return action;
    } else {
      return {
        type: type,
        payload: action
      };
    }
  };
};

commonReducer = function(TYPES, DEFAULT) {
  var reducers;
  reducers = {};
  reducers[TYPES.UPDATE || 'UPDATE'] = function(state, action) {
    return {...state, ...action.payload};
  };
  reducers[TYPES.RESET || 'RESET'] = function(state, action) {
    return DEFAULT;
  };
  return reducers;
};

initialReducer = function(TYPES) {
  var reducers;
  reducers = {};
  reducers[TYPES.INIT || 'INIT'] = function(state, action) {
    return {
      ...state,
      inited: true
    };
  };
  return reducers;
};

createReducer = function(DEFAULT = {}, ...mixins) {
  var i, len, mixin, reducers;
  reducers = {};
  for (i = 0, len = mixins.length; i < len; i++) {
    mixin = mixins[i];
    reducers = {...reducers, ...mixin};
  }
  return function(state = DEFAULT, action) {
    var reducer;
    reducer = reducers[action.type];
    if (reducer) {
      return reducer(state, action);
    } else {
      return state;
    }
  };
};

createActions = function(prefix, ...creators) {
  var actionCreator, actionType, created, creator, i, len, name, type;
  created = {
    TYPES: {}
  };
  for (i = 0, len = creators.length; i < len; i++) {
    creator = creators[i];
    for (name in creator) {
      actionCreator = creator[name];
      actionType = type = toSnakeCase(name).toUpperCase();
      if (prefix) {
        actionType = `${prefix}${DEFAULTS.JOIN}${actionType}`;
      }
      if (!isFunction(actionCreator)) {
        actionCreator = identity;
      }
      actionCreator = wrapCreator(actionType, actionCreator.bind(created));
      created.TYPES[type] = actionType;
      created[name] = actionCreator;
    }
  }
  return created;
};

skipMiddleware = function(store) {
  return function(next) {
    return function(action) {
      if (action.type !== null) {
        return next(action);
      }
    };
  };
};

reducerEnhancer = function(createStore) {
  return function(reducer, initialState, enhancer) {
    var enhancedReducer, queue, schedule, store;
    queue = [];
    schedule = function(action) {
      return queue.push(action);
    };
    enhancedReducer = function(state, action) {
      state = reducer(state, action);
      if (isFunction(state)) {
        state = state(schedule, store.getState);
      }
      return state;
    };
    store = createStore(enhancedReducer, initialState, enhancer);
    return {
      ...store,
      dispatch: function(action) {
        var next;
        action = store.dispatch(action);
        while (next = queue.shift()) {
          action = store.dispatch(next);
        }
        return action;
      }
    };
  };
};

combineReducers = function(reducers) {
  return function(state = {}, action) {
    var callbacks, hasChanged, k, nextReducerState, nextState, previousReducerState, r;
    hasChanged = false;
    nextState = {};
    callbacks = [];
    for (k in reducers) {
      r = reducers[k];
      previousReducerState = state[k];
      nextReducerState = r(previousReducerState, action);
      nextState[k] = nextReducerState;
      if (isFunction(nextReducerState)) {
        callbacks.push(k);
        continue;
      }
      hasChanged = hasChanged || nextReducerState !== previousReducerState;
    }
    if (callbacks.length) {
      return function(schedule, getState) {
        var i, len;
        for (i = 0, len = callbacks.length; i < len; i++) {
          k = callbacks[i];
          nextState[k] = nextState[k](schedule, getState);
        }
        return nextState;
      };
    }
    if (hasChanged) {
      return nextState;
    } else {
      return state;
    }
  };
};

module.exports = {SKIP, DEFAULTS, skipMiddleware, reducerEnhancer, combineReducers, createActions, createReducer, commonReducer, initialReducer};
